@@ -1,6 +1,10 @@
 #!/usr/bin/env python
-"""Overlay the histograms from several root files with identical structure"""
-__version__ = '1.1'
+"""
+Draw histograms to image files, specifying options on the command line.
+
+You can overlay plots from multiple ROOT files with identical structure.
+"""
+__version__ = '1.2'
 
 ## Created by Jeff Klukas (klukas@wisc.edu), November 2009
 ## Updated February 2010
@@ -55,8 +59,6 @@ next_counter = counter_generator().next
 def process_directory(path, files):
     """Loop through all histograms in the directory and plot them."""
     dir_to_make = '%s/%s' % (options.plot_dir, path)
-    if not os.path.exists(dir_to_make):
-        os.mkdir(dir_to_make)
     keys = files[0].file.GetDirectory(path).GetListOfKeys()
     key = keys[0]
     while key:
@@ -71,6 +73,8 @@ def process_directory(path, files):
                            not obj.IsA().InheritsFrom('TH2') and
                            not obj.IsA().InheritsFrom('TH3'))
         if (matches_path and is_1D_histogram):
+            if not os.path.exists(dir_to_make):
+                os.makedirs(dir_to_make)
             if options.use_matplotlib:
                 process_hist_matplotlib(path, new_path, files, obj)
             else:
@@ -210,14 +214,24 @@ def process_hist_matplotlib(path, new_path, files, obj):
     stack = r2m.HistStack()
     #### Apply options to hist from each file
     for i, file in enumerate(files):
-        hist = r2m.Hist(file.file.GetDirectory(path).Get(name),
-                        file.name, xlabel=options.xlabel, ylabel=options.ylabel)
+        hist = r2m.Hist(file.file.GetDirectory(path).Get(name), 
+                        xlabel=options.xlabel, ylabel=options.ylabel)
+        if not hist: continue
         if options.title:
             if options.title[0] == "+":
                 hist.title += options.title[1:]
             else:
                 hist.title = options.title
-        if not hist: continue
+        if options.xlabel:
+            if options.xlabel[0] == "+":
+                hist.xlabel += options.xlabel[1:]
+            else:
+                hist.xlabel = options.xlabel
+        if options.ylabel:
+            if options.ylabel[0] == "+":
+                hist.ylabel += options.ylabel[1:]
+            else:
+                hist.ylabel = options.ylabel
         if options.efficiency_from:
             denom = r2m.Hist(file.file.Get(options.efficiency_from))
             eff, up, down = wilson([sum(hist.y)], [sum(denom.y)])
@@ -225,19 +239,25 @@ def process_hist_matplotlib(path, new_path, files, obj):
                            (eff[0] * 100., max(up[0], down[0]) * 100.))
             hist.y, up, down = wilson(hist.y, denom.y)
             hist.yerr = np.array([up, down])
-        stack.add(hist, color=options.colors[i], fmt=options.marker_styles[i])
+        if options.area_normalize or (options.sticky and 'Norm' in name):
+            hist.y /= sum(hist.y)
+        if options.markers: fmt = options.marker_styles[i]
+        else:               fmt = 'o'
+        stack.add(hist, label=file.name, color=options.colors[i], fmt=fmt)
     if (options.errorbar):
-        stack.plot_errorbar(replace=options.replace, yerr=True)
+        plots = stack.plot_errorbar(replace=options.replace, yerr=True)
     if options.stack:
         stack.plot_barstack(replace=options.replace)
     if options.bar:
         stack.plot_bar(replace=options.replace, alpha=0.5)
     if options.efficiency:
         plt.ylim(0, 1)
-    plt.legend(numpoints=1, loc=options.legend)
+    if options.logx: ax.set_xscale('log')
+    if options.logy: ax.set_yscale('log')
     if options.numbering:
-        plt.text(1.0, 1.025, counter, transform=ax.transAxes,
-                 horizontalalignment='right')
+        plt.text(0.98, 0.98, counter, transform=plt.figure(1).transFigure,
+                 size="small", ha='right', va='top')
+    plt.legend(numpoints=1, loc=options.legend)
     output_file_name = '%s/%s/%s' % (options.plot_dir, path, name)
     plt.savefig(output_file_name, transparent=options.transparent)
     if options.ext == "pdf":
@@ -268,7 +288,7 @@ def merge_pdf():
     if "001.pdf" not in os.listdir('.'):
         print "No output files, so no merged pdf was made"
         return
-    print "Writing merged pdf..."
+    print "Writing %s.pdf..." % options.output
     os.system('gs -q -dBATCH -dNOPAUSE -sDEVICE=pdfwrite '
               '-dAutoRotatePages=/All '
               '-sOutputFile=%s.pdf ' % options.output +
@@ -356,12 +376,21 @@ def wilson(numerator, denominator):
     return eff, up, down
 
 def add_options_from_configuration(options, nfiles):
-    """Define a set of global variables storing style information, etc."""
-    exec("import %s as config" % options.config[:-3])
-    for attribute in dir(config):
-        if '__' not in attribute:
-            attr = getattr(config, attribute)
-            setattr(options, attribute, attr)
+    """Define a set of global variables storing style information"""
+    def append_to_options(config, options):
+        for attribute in dir(config):
+            if '__' not in attribute:
+                attr = getattr(config, attribute)
+                setattr(options, attribute, attr)
+        return options
+    try:
+        from root2matplot import overlayHists_config
+        options = append_to_options(overlayHists_config, options)
+    except: pass
+    try:
+        import overlayHists_config
+        options = append_to_options(overlayHists_config, options)
+    except: pass
     return options
 
 def process_options(options, nfiles):
@@ -422,16 +451,27 @@ def process_options(options, nfiles):
 def main():
     usage="""usage: %prog [options] file1.root file2.root file3.root ...
 
+Documentation: http://packages.python.org/root2matplot/
+
 Function: overlays corresponding histograms from several files, dumping the
   images into a directory structure that mirrors that of the ROOT file and,
   if output format is pdf, also merging all images into a single file;
-  most style options can be controlled from your rootlogon.C macro"""
+  most style options can be controlled from your rootlogon.C macro
+
+Note: By default, this script will work completely in ROOT.  If you use the
+  options 'bar', 'errorbar', or 'stack', output will be produced in matplotlib.
+  More complex style attributes like colors and markers used can be configured
+  by placing an overlayHists_config.py in your current directory."""
     
     parser = optparse.OptionParser(usage=usage, 
                                    version='%s %s' % ('%prog', __version__))
+    group2 = optparse.OptionGroup(parser,
+                                "options available only with ROOT output")
+    group3 = optparse.OptionGroup(parser,
+                                "options available only with matplotlib output")
     parser.add_option('-e', '--ext', default='pdf', 
                       help="choose an output extension; default is pdf")
-    parser.add_option('--draw', default='nostack p H',
+    group2.add_option('--draw', default='nostack p H',
                       help="pass DRAW to ROOT's Draw command; default is "
                       "'nostack p H'; try 'nostack e' for error bars")
     parser.add_option('-m', '--markers', action='store_true', default=False,
@@ -459,7 +499,7 @@ Function: overlays corresponding histograms from several files, dumping the
                       help="output a matplotlib errorbar graph")
     parser.add_option('--bar', action="store_true", default=False,
                       help="output a matplotlib bar graph")
-    parser.add_option('--transparent', action="store_true", default=False,
+    group3.add_option('--transparent', action="store_true", default=False,
                       help="use a transparent background for matplotlib "
                       "output")
     parser.add_option('--colormap', default=None,
@@ -472,7 +512,7 @@ Function: overlays corresponding histograms from several files, dumping the
                       help="Place legend in LOC, according to matplotlib "
                       "location codes; for ROOT, you can use 1 (default) for "
                       "upper right or 8 for lower center")
-    parser.add_option('--size', default='6x4.5',
+    group3.add_option('--size', default='6x4.5',
                       help="Define the size of the matplotlib plot as "
                       "width x height in inches; default is '6x4.5'")
     parser.add_option('--title', default=None,
@@ -481,9 +521,10 @@ Function: overlays corresponding histograms from several files, dumping the
                       help="Replace the x-axis label with XLABEL")
     parser.add_option('--ylabel', default=None,
                       help="Replace the y-axis label with YLABEL")
-    parser.add_option('--config', default="overlayHists_config.py",
-                      help="Use the given configuration file; default is "
-                      "overlayHists_config.py")
+##     parser.add_option('--config', default=None,
+##                       help="Use the given style configuration file; default "
+##                       "styles are taken from overlayHists_config.py in the "
+##                       "root2matplot distribution or the current directory")
     parser.add_option('--efficiency-from', default=None, metavar="DENOM",
                       help="Divide all plots by DENOM")
     group1 = optparse.OptionGroup(
@@ -495,6 +536,7 @@ Function: overlays corresponding histograms from several files, dumping the
         "to turn on sticky keywords (such as 'Norm').  Any plot that includes "
         "the given keyword in its ROOT name will have the option applied "
         "regardless of its presence or absence on the command line."
+        "The last three options are currently only available in ROOT output."
         )
     group1.add_option('-n', '--area-normalize', action='store_true',
                       default=False,
@@ -514,6 +556,8 @@ Function: overlays corresponding histograms from several files, dumping the
     group1.add_option('--ratio', action='store_true', default=False,
                       help="'Ratio': display a ratio plot below the normal "
                       "plot")
+    parser.add_option_group(group2)
+    parser.add_option_group(group3)
     parser.add_option_group(group1)
     global options
     options, arguments = parser.parse_args()
